@@ -19,34 +19,34 @@ from robotcode.core.lsp.types import (
     Range,
 )
 from robotcode.core.uri import Uri
-from robotcode.language_server.robotframework.parts.model_helper import ModelHelperMixin
-from robotcode.language_server.robotframework.utils.ast_utils import (
+
+from ..utils.ast_utils import (
     HasTokens,
     Statement,
     Token,
     is_not_variable_token,
-    iter_over_keyword_names_and_owners,
     range_from_node,
     range_from_node_or_token,
     range_from_token,
     strip_variable_token,
     tokenize_variables,
 )
-from robotcode.language_server.robotframework.utils.async_ast import AsyncVisitor
-from robotcode.language_server.robotframework.utils.version import get_robot_version
-
+from ..utils.async_ast import AsyncVisitor
+from ..utils.version import get_robot_version
 from .entities import (
+    ArgumentDefinition,
     CommandLineVariableDefinition,
     EnvironmentVariableDefinition,
     LibraryEntry,
-    ResourceEntry,
+    LocalVariableDefinition,
     VariableDefinition,
     VariableDefinitionType,
     VariableNotFoundDefinition,
 )
-from .library_doc import KeywordDoc, KeywordMatcher, is_embedded_keyword
+from .errors import DIAGNOSTICS_SOURCE_NAME, Error
+from .library_doc import KeywordDoc, is_embedded_keyword
+from .model_helper import ModelHelperMixin
 from .namespace import (
-    DIAGNOSTICS_SOURCE_NAME,
     KeywordFinder,
     Namespace,
 )
@@ -57,6 +57,7 @@ class AnalyzerResult:
     diagnostics: List[Diagnostic]
     keyword_references: Dict[KeywordDoc, Set[Location]]
     variable_references: Dict[VariableDefinition, Set[Location]]
+    local_variable_assignments: Dict[VariableDefinition, Set[Range]]
     namespace_references: Dict[LibraryEntry, Set[Location]]
 
 
@@ -67,17 +68,15 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
         namespace: Namespace,
         finder: KeywordFinder,
         ignored_lines: List[int],
-        libraries_matchers: Dict[KeywordMatcher, LibraryEntry],
-        resources_matchers: Dict[KeywordMatcher, ResourceEntry],
     ) -> None:
         from robot.parsing.model.statements import Template, TestTemplate
 
+        super().__init__()
+
         self.model = model
         self.namespace = namespace
         self.finder = finder
         self._ignored_lines = ignored_lines
-        self.libraries_matchers = libraries_matchers
-        self.resources_matchers = resources_matchers
 
         self.current_testcase_or_keyword_name: Optional[str] = None
         self.test_template: Optional[TestTemplate] = None
@@ -86,6 +85,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
         self._diagnostics: List[Diagnostic] = []
         self._keyword_references: Dict[KeywordDoc, Set[Location]] = defaultdict(set)
         self._variable_references: Dict[VariableDefinition, Set[Location]] = defaultdict(set)
+        self._local_variable_assignments: Dict[VariableDefinition, Set[Range]] = defaultdict(set)
         self._namespace_references: Dict[LibraryEntry, Set[Location]] = defaultdict(set)
 
     async def run(self) -> AnalyzerResult:
@@ -95,7 +95,11 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
         await self.visit(self.model)
 
         return AnalyzerResult(
-            self._diagnostics, self._keyword_references, self._variable_references, self._namespace_references
+            self._diagnostics,
+            self._keyword_references,
+            self._variable_references,
+            self._local_variable_assignments,
+            self._namespace_references,
         )
 
     def yield_argument_name_and_rest(self, node: ast.AST, token: Token) -> Iterator[Token]:
@@ -195,7 +199,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
             )
 
             if isinstance(node, KeywordCall) and node.keyword:
-                kw_doc = self.finder.find_keyword(node.keyword)
+                kw_doc = self.finder.find_keyword(node.keyword, raise_keyword_error=False)
                 if kw_doc is not None and kw_doc.longname in ["BuiltIn.Comment"]:
                     severity = DiagnosticSeverity.HINT
 
@@ -225,7 +229,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                                     message=f"Variable '{var.name}' not found.",
                                     severity=severity,
                                     source=DIAGNOSTICS_SOURCE_NAME,
-                                    code="VariableNotFound",
+                                    code=Error.VARIABLE_NOT_FOUND,
                                 )
                             else:
                                 if isinstance(var, EnvironmentVariableDefinition) and var.default_value is None:
@@ -236,16 +240,14 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                                             message=f"Environment variable '{var.name}' not found.",
                                             severity=severity,
                                             source=DIAGNOSTICS_SOURCE_NAME,
-                                            code="EnvirommentVariableNotFound",
+                                            code=Error.ENVIROMMENT_VARIABLE_NOT_FOUND,
                                         )
 
                                 if self.namespace.document is not None:
                                     if isinstance(var, EnvironmentVariableDefinition):
                                         var_token.value, _, _ = var_token.value.partition("=")
 
-                                        var_range = range_from_token(var_token)
-                                    else:
-                                        var_range = range_from_token(var_token)
+                                    var_range = range_from_token(var_token)
 
                                     suite_var = None
                                     if isinstance(var, CommandLineVariableDefinition):
@@ -267,6 +269,10 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                                             self._variable_references[suite_var].add(
                                                 Location(self.namespace.document.document_uri, var_range)
                                             )
+                                        if token1.type in [RobotToken.ASSIGN] and isinstance(
+                                            var, (LocalVariableDefinition, ArgumentDefinition)
+                                        ):
+                                            self._local_variable_assignments[var].add(var_range)
 
                                     elif var not in self._variable_references and token1.type in [
                                         RobotToken.ASSIGN,
@@ -296,7 +302,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                             message=f"Variable '{var.name}' not found.",
                             severity=DiagnosticSeverity.ERROR,
                             source=DIAGNOSTICS_SOURCE_NAME,
-                            code="VariableNotFound",
+                            code=Error.VARIABLE_NOT_FOUND,
                         )
                     else:
                         if self.namespace.document is not None:
@@ -389,57 +395,40 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
             lib_range = None
             kw_namespace = None
 
+            result = self.finder.find_keyword(keyword, raise_keyword_error=False)
+
             if keyword is not None:
-                for lib, name in iter_over_keyword_names_and_owners(keyword):
-                    if (
-                        lib is not None
-                        and not any(k for k in self.libraries_matchers.keys() if k == lib)
-                        and not any(k for k in self.resources_matchers.keys() if k == lib)
-                    ):
-                        continue
+                lib_entry, kw_namespace = await self.get_namespace_info_from_keyword_token(
+                    self.namespace, keyword_token
+                )
 
-                    lib_entry, kw_namespace = await self.get_namespace_info_from_keyword(
-                        self.namespace, keyword_token, self.libraries_matchers, self.resources_matchers
-                    )
-                    if lib_entry and kw_namespace:
-                        r = range_from_token(keyword_token)
-                        lib_range = r
-                        r.end.character = r.start.character + len(kw_namespace)
-                        kw_range.start.character = r.end.character + 1
-                        lib_range.end.character = kw_range.start.character - 1
-
-            result = self.finder.find_keyword(keyword)
+                if lib_entry and kw_namespace:
+                    r = range_from_token(keyword_token)
+                    lib_range = r
+                    r.end.character = r.start.character + len(kw_namespace)
+                    kw_range.start.character = r.end.character + 1
+                    lib_range.end.character = kw_range.start.character - 1
 
             if (
                 result is not None
                 and lib_entry is not None
                 and kw_namespace
                 and result.parent is not None
-                and result.parent != lib_entry.library_doc.digest
+                and result.parent != lib_entry.library_doc
             ):
-                entry = next(
-                    (
-                        v
-                        for v in (await self.namespace.get_libraries()).values()
-                        if v.library_doc.digest == result.parent
-                    ),
-                    None,
-                )
-                if entry is None:
-                    entry = next(
-                        (
-                            v
-                            for v in (await self.namespace.get_resources()).values()
-                            if v.library_doc.digest == result.parent
-                        ),
-                        None,
-                    )
-                if entry is not None:
-                    lib_entry = entry
+                lib_entry = None
+                kw_range = range_from_token(keyword_token)
 
             if kw_namespace and lib_entry is not None and lib_range is not None:
                 if self.namespace.document is not None:
-                    self._namespace_references[lib_entry].add(Location(self.namespace.document.document_uri, lib_range))
+                    entries = [lib_entry]
+                    if self.finder.multiple_keywords_result is not None:
+                        entries = next(
+                            (v for k, v in (await self.namespace.get_namespaces()).items() if k == kw_namespace),
+                            entries,
+                        )
+                    for entry in entries:
+                        self._namespace_references[entry].add(Location(self.namespace.document.document_uri, lib_range))
 
             if not ignore_errors_if_contains_variables or is_not_variable_token(keyword_token):
                 for e in self.finder.diagnostics:
@@ -452,8 +441,8 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
 
             if result is None:
                 if self.namespace.document is not None and self.finder.multiple_keywords_result is not None:
-                    for lib_entry, kw_doc in self.finder.multiple_keywords_result:
-                        self._keyword_references[kw_doc].add(Location(self.namespace.document.document_uri, kw_range))
+                    for d in self.finder.multiple_keywords_result:
+                        self._keyword_references[d].add(Location(self.namespace.document.document_uri, kw_range))
             else:
                 if self.namespace.document is not None:
                     self._keyword_references[result].add(Location(self.namespace.document.document_uri, kw_range))
@@ -507,21 +496,21 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                         f"{f': {result.deprecated_message}' if result.deprecated_message else ''}.",
                         severity=DiagnosticSeverity.HINT,
                         tags=[DiagnosticTag.DEPRECATED],
-                        code="DeprecatedKeyword",
+                        code=Error.DEPRECATED_KEYWORD,
                     )
                 if result.is_error_handler:
                     self.append_diagnostics(
                         range=kw_range,
                         message=f"Keyword definition contains errors: {result.error_handler_message}",
                         severity=DiagnosticSeverity.ERROR,
-                        code="KeywordContainsErrors",
+                        code=Error.KEYWORD_CONTAINS_ERRORS,
                     )
                 if result.is_reserved():
                     self.append_diagnostics(
                         range=kw_range,
                         message=f"'{result.name}' is a reserved keyword.",
                         severity=DiagnosticSeverity.ERROR,
-                        code="ReservedKeyword",
+                        code=Error.RESERVED_KEYWORD,
                     )
 
                 if get_robot_version() >= (6, 0) and result.is_resource_keyword and result.is_private():
@@ -531,13 +520,13 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                             message=f"Keyword '{result.longname}' is private and should only be called by"
                             f" keywords in the same file.",
                             severity=DiagnosticSeverity.WARNING,
-                            code="PrivateKeyword",
+                            code=Error.PRIVATE_KEYWORD,
                         )
 
                 if not isinstance(node, (Template, TestTemplate)):
                     try:
-                        if result.arguments is not None:
-                            result.arguments.resolve(
+                        if result.arguments_spec is not None:
+                            result.arguments_spec.resolve(
                                 [v.value for v in argument_tokens],
                                 None,
                                 resolve_variables_until=result.args_to_process,
@@ -595,7 +584,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                                 range=range_from_token(var_token),
                                 message=f"Variable '{var.name}' not found.",
                                 severity=DiagnosticSeverity.ERROR,
-                                code="VariableNotFound",
+                                code=Error.VARIABLE_NOT_FOUND,
                             )
                         else:
                             if self.namespace.document is not None:
@@ -618,7 +607,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                     name, value = split_from_equals(arg.value)
                     if value is not None and name:
                         arg_def = next(
-                            (e for e in result.argument_definitions if e.name_token and e.name_token.value == name),
+                            (e for e in result.argument_definitions if e.name[2:-1] == name),
                             None,
                         )
                         if arg_def is not None:
@@ -675,7 +664,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                         range=range_from_token(t),
                         message=f"Incorrect use of {t.value}.",
                         severity=DiagnosticSeverity.ERROR,
-                        code="IncorrectUse",
+                        code=Error.INCORRECT_USE,
                     )
                     continue
 
@@ -843,7 +832,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                 range=range_from_node_or_token(value, value.get_token(RobotToken.ASSIGN)),
                 message="Keyword name cannot be empty.",
                 severity=DiagnosticSeverity.ERROR,
-                code="KeywordNameEmpty",
+                code=Error.KEYWORD_NAME_EMPTY,
             )
         else:
             await self._analyze_keyword_call(
@@ -856,7 +845,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                 message="Code is unreachable.",
                 severity=DiagnosticSeverity.HINT,
                 tags=[DiagnosticTag.UNNECESSARY],
-                code="CodeUnreachable",
+                code=Error.CODE_UNREACHABLE,
             )
 
         await self.generic_visit(node)
@@ -874,7 +863,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                 range=range_from_node_or_token(testcase, name_token),
                 message="Test case name cannot be empty.",
                 severity=DiagnosticSeverity.ERROR,
-                code="TestCaseNameEmpty",
+                code=Error.TESTCASE_NAME_EMPTY,
             )
 
         self.current_testcase_or_keyword_name = testcase.name
@@ -907,7 +896,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                     range=range_from_node_or_token(keyword, name_token),
                     message="Keyword cannot have both normal and embedded arguments.",
                     severity=DiagnosticSeverity.ERROR,
-                    code="KeywordNormalAndEmbbededError",
+                    code=Error.KEYWORD_CONTAINS_NORMAL_AND_EMBBEDED_ARGUMENTS,
                 )
         else:
             name_token = cast(KeywordName, keyword.header).get_token(RobotToken.KEYWORD_NAME)
@@ -915,7 +904,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                 range=range_from_node_or_token(keyword, name_token),
                 message="Keyword name cannot be empty.",
                 severity=DiagnosticSeverity.ERROR,
-                code="KeywordNameEmpty",
+                code=Error.KEYWORD_NAME_EMPTY,
             )
 
         self.current_testcase_or_keyword_name = keyword.name
@@ -953,8 +942,8 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
             result = self.finder.find_keyword(keyword)
             if result is not None:
                 try:
-                    if result.arguments is not None:
-                        result.arguments.resolve(
+                    if result.arguments_spec is not None:
+                        result.arguments_spec.resolve(
                             args,
                             None,
                             resolve_variables_until=result.args_to_process,
@@ -992,7 +981,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                     message="`Force Tags` is deprecated in favour of new `Test Tags` setting.",
                     severity=DiagnosticSeverity.INFORMATION,
                     tags=[DiagnosticTag.DEPRECATED],
-                    code="DeprecatedHyphenTag",
+                    code=Error.DEPRECATED_HYPHEN_TAG,
                 )
 
     async def visit_DefaultTags(self, node: ast.AST) -> None:  # noqa: N802
@@ -1006,7 +995,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                 message="`Force Tags` is deprecated in favour of new `Test Tags` setting.",
                 severity=DiagnosticSeverity.INFORMATION,
                 tags=[DiagnosticTag.DEPRECATED],
-                code="DeprecatedHyphenTag",
+                code=Error.DEPRECATED_HYPHEN_TAG,
             )
 
     async def visit_Tags(self, node: ast.AST) -> None:  # noqa: N802
@@ -1026,7 +1015,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                         f"literal value and to avoid this warning.",
                         severity=DiagnosticSeverity.WARNING,
                         tags=[DiagnosticTag.DEPRECATED],
-                        code="DeprecatedHyphenTag",
+                        code=Error.DEPRECATED_HYPHEN_TAG,
                     )
 
     def _check_import_name(self, value: Optional[str], node: ast.AST, type: str) -> None:
@@ -1035,7 +1024,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
                 range=range_from_node(node),
                 message=f"{type} setting requires value.",
                 severity=DiagnosticSeverity.ERROR,
-                code="ImportRequiresValue",
+                code=Error.IMPORT_REQUIRES_VALUE,
             )
 
     async def visit_VariablesImport(self, node: ast.AST) -> None:  # noqa: N802
@@ -1051,7 +1040,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
         if name_token is None:
             return
 
-        entries = await self.namespace.get_imported_variables()
+        entries = await self.namespace.get_import_entries()
         if entries and self.namespace.document:
             for v in entries.values():
                 if v.import_source == self.namespace.source and v.import_range == range_from_token(name_token):
@@ -1071,7 +1060,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
         if name_token is None:
             return
 
-        entries = await self.namespace.get_resources()
+        entries = await self.namespace.get_import_entries()
         if entries and self.namespace.document:
             for v in entries.values():
                 if v.import_source == self.namespace.source and v.import_range == range_from_token(name_token):
@@ -1091,7 +1080,7 @@ class Analyzer(AsyncVisitor, ModelHelperMixin):
         if name_token is None:
             return
 
-        entries = await self.namespace.get_libraries()
+        entries = await self.namespace.get_import_entries()
         if entries and self.namespace.document:
             for v in entries.values():
                 if v.import_source == self.namespace.source and v.import_range == range_from_token(name_token):
