@@ -26,24 +26,24 @@ from robotcode.core.lsp.types import (
     WorkspaceEdit,
 )
 from robotcode.core.utils.inspect import iter_methods
-from robotcode.language_server.common.decorators import code_action_kinds, language_id
-from robotcode.language_server.common.text_document import TextDocument
-from robotcode.language_server.robotframework.utils import ast_utils
-from robotcode.language_server.robotframework.utils.ast_utils import (
+from robotcode.robot.utils import get_robot_version
+
+from ...common.decorators import code_action_kinds, language_id
+from ...common.text_document import TextDocument
+from ..diagnostics.model_helper import ModelHelperMixin
+from ..utils import ast_utils
+from ..utils.ast_utils import (
     BodyBlock,
     get_node_at_position,
     get_nodes_at_position,
     range_from_node,
     range_from_token,
 )
-from robotcode.robot.utils import get_robot_version
-
 from .code_action_helper_mixin import SHOW_DOCUMENT_SELECT_AND_RENAME_COMMAND, CodeActionDataBase, CodeActionHelperMixin
-from .model_helper import ModelHelperMixin
 from .protocol_part import RobotLanguageServerProtocolPart
 
 if TYPE_CHECKING:
-    from robotcode.language_server.robotframework.protocol import RobotLanguageServerProtocol  # pragma: no cover
+    from ..protocol import RobotLanguageServerProtocol  # pragma: no cover
 
 
 class SurroundType(Enum):
